
/*
 * @(#)Naming.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

import java.util.*;

/**
 * Shared Data Naming class.
 *
 * This class provides simple URL based naming for Shared Data objects.
 *
 * Based on the RMI Naming class.
 *
 * @version     0.89 28 January 1997
 * @author      Rich Burridge
 */

public final class
Naming extends SharedDataObject {

    /** Default Shared Data registry port number. */
    static final int REGISTRY_PORT = 4562;

    /** The Naming class instances for the various hosts. */
    private static Hashtable connections = null;

    /** Handle to Naming class proxy. */
    public AbstractNamingProxy po ;


/* Disallow anyone from creating one of these. */

    private
    Naming(String host, int port, String connectType) {
	if (Naming_Debug) {
            System.out.println("Naming: constructor:" +
			       " host: " + host +
			       " port: " + port +
			       " connection type: " + connectType);
        }

        try {
            String className = "SharedData." + connectType + ".NamingProxy";
		System.out.println("className variable is "+className);

            po = (AbstractNamingProxy) Class.forName(className).newInstance();
            po.initProxy(host, port);
        } catch (Exception e) {
            System.out.println("Naming_Debug: constructor exception " + e);
            if (Naming_Debug) {
                e.printStackTrace();
            }
        }
    }


/**
 * <A NAME="SD_BIND"></A>
 * <EM>bind</EM> binds the name to the specified Shared Data object.
 *
 * The name should be of the form:
 *     coll://<machine>[:<port>]/<objecttype>/<objectname>
 *
 * where valid object types are "Session" and "Client".
 *
 * So for example:
 *     bind("coll://stard:3355/Session/chat.session", chatSession);
 *
 *     bind("coll://jsda:4386/Client/fred.client", fredClient);
 *
 * @param <EM>name</EM> the name associated with this object.
 * @param <EM>object</EM> the object to bind.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * The following exception types can occur:
 * <PRE>
 *      INVALID_URL    - url string given is invalid.
 *      ALREADY_BOUND  - this url string is already bound to an object.
 * </PRE>
 */

    public static void
    bind(String name, Object object) throws SharedDataException {
	URLString urlString = new URLString(name);

        if (Naming_Debug) {
            System.out.println("Naming: bind:"     +
				" name: "   + name +
				" object: " + object);
        }

        if (urlString.getStatus()) {
            String host        = urlString.getHost();
            String connectType = urlString.getConnectionType();
	    Naming naming      = getRegistryConnection(host, connectType);

	    naming.po.bind(naming, urlString, name, object);
	} else {
            throw new SharedDataException(SharedDataException.INVALID_URL);
	}
    }


/**
 * <A NAME="SD_REBIND"></A>
 * <EM>rebind</EM> rebinds the name to a new object, replacing any existing
 * binding.
 *
 * @param <EM>name</EM> the name associated with this new object.
 * @param <EM>object</EM> the object to rebind.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * The following exception types can occur:
 * <PRE>
 *      INVALID_URL - url string given is invalid.
 * </PRE>
 */

    public static void
    rebind(String name, Object object) throws SharedDataException {
        URLString urlString = new URLString(name);

	if (Naming_Debug) {
            System.out.println("Naming: rebind:"   +
				" name: "   + name +
				" object: " + object);
        }

        if (urlString.getStatus()) {
            String host        = urlString.getHost();
            String connectType = urlString.getConnectionType();
            Naming naming      = getRegistryConnection(host, connectType);

            naming.po.rebind(naming, urlString, name, object);
        } else {    
            throw new SharedDataException(SharedDataException.INVALID_URL);
        }
    }


/**
 * <A NAME="SD_UNBIND"></A>
 * <EM>unbind</EM> unbinds the object associated with this name.
 *
 * @param <EM>name</EM> the name of the object to unbind.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      INVALID_URL - url string given is invalid.
 *      NOT_BOUND   - no object bound to this url string.
 * </PRE>
 */
 
    public static void
    unbind(String name, Object object) throws SharedDataException {
        URLString urlString = new URLString(name);

	if (Naming_Debug) {
            System.out.println("Naming: unbind:"  +
			       " name: "   + name +
			       " object: " + object);
        }
 
        if (urlString.getStatus()) {
            String host        = urlString.getHost();
            String connectType = urlString.getConnectionType();
            Naming naming      = getRegistryConnection(host, connectType);

            naming.po.unbind(naming, urlString, name, object);
	} else {
            throw new SharedDataException(SharedDataException.INVALID_URL);
        }
    }


/**
 * <A NAME="SD_LOOKUP"></A>
 * <EM>lookup</EM> returns the Shared Data object for the given name.
 *
 * The name should be of the form:
 *     coll://<machine>[:<port>]/<objecttype>/<objectname>
 *
 * where valid object types are "Session" and "Client".
 *
 * So for example:
 *     Session session = lookup("coll://stard:3355/Session/chat.session");
 *
 *     Client  client  = lookup("coll://jsda:4386/Client/fred.client");
 *
 * @param <EM>name</EM> the name of the object to find.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      INVALID_URL - url string given is invalid.
 *      NOT_BOUND   - no object bound to this url string.
 * </PRE>
 *
 * @return the resolved object.
 */

    public static Object
    lookup(String name) throws SharedDataException {
      System.out.println("Inside Naming's lookup method "+"and the name variable is "+name);
        URLString urlString = new URLString(name);
      System.out.println("url string in naming's lookup method created "+urlString);
	Object    object    = null;

        if (Naming_Debug) {
            System.out.println("Naming: lookup:" +
			       " name: " + name);
        }

        if (urlString.getStatus()) {
            System.out.println("Inside lookup - urlstring-getstatus method");
            String host        = urlString.getHost();
            System.out.println("host in getstatus "+host);
            String connectType = urlString.getConnectionType();
            System.out.println("connection type in getstatus method "+connectType);
            Naming naming      = getRegistryConnection(host, connectType);
            System.out.println("After getRegistryConnection... "+naming);

            object = naming.po.lookup(naming, urlString, name);
            System.out.println("After naming.po.lookup");
	} else { 
            throw new SharedDataException(SharedDataException.INVALID_URL);
        }

	return(object);
    }


/**
 * <A NAME="SD_LIST"></A>
 * <EM>list</EM> lists all the names of the known bound Shared Data objects.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      <currently none>
 * </PRE>
 *
 * @return an array of Strings of the names of all the known bound Shared
 * Data objects.
 */

    public static String[]
    list() throws SharedDataException {
        int      total    = 0;
	Vector   allNames = new Vector();
	String[] names    = null;

        if (Naming_Debug) {
            System.out.println("Naming: list.");
        }

        for (Enumeration e = connections.elements(); e.hasMoreElements();) {
            Naming   naming   = (Naming) e.nextElement();
	    String[] newNames = naming.po.list(naming);

	    for (int i = 0; i < newNames.length; i++) {
                allNames.addElement(newNames[i]);
	    }

	    total += newNames.length;
	}

        names = new String[total];
        allNames.copyInto(names);
        return(names);
    }


/**
 * <A NAME="SD_GETREGISTRYCONNECTION"></A>
 * <EM>getRegistryConnection</EM>
 *
 * @param <EM>host</EM>
 */
 
    private static Naming
    getRegistryConnection(String host, String connectType) {
        Naming naming = null;

        if (Naming_Debug) {
            System.out.println("Naming: getRegistryConnection:" +
			       " host: "            + host +
			       " connection type: " + connectType);
        }

        if (connections == null) {
            connections = new Hashtable();
            System.out.println("New hashtable for maintaining reco of all server connections in registry created");
        }

        if ((naming = (Naming) connections.get(host)) == null) {
            naming = new Naming(host, Naming.REGISTRY_PORT, connectType);

            connections.put(host, naming);
            System.out.println("HOST "+host+" "+"mapped to value "+naming+" "+"in the hashtable");
        }
        return(naming);
    }
}
