
/*
 * @(#)Session.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

import SharedData.primitive.SharedByteArrayObserver;

/**
 * Shared Data Session class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

public abstract class
Session extends Manageable {

    /** To signify a manageable object of type Session. */
    public static final String M_Session = "Session";

    /** Handle to client-side proxy. */
    public AbstractSessionProxy po;

    /** Handle to server-side object. */
    public AbstractSessionServer so;

    /** The number of channels currently created for this session. */
    private Integer noChannels = new Integer(0);

    /** The number of tokens currently created for this session. */
    private Integer noTokens = new Integer(0);


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM> joins a client to this session. This client will now be
 * known to this session. Other clients in the session will be sent a
 * <EM>sessionChanged</EM> message, when this happens.
 *
 * @param <EM>client</EM> the client in question.
 *
 * @exception SharedDataException If a Shared Data exception has occured. 
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION   - session given does not exist.
 *      NAME_IN_USE       - this client name already exists.
 *      PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */

    public void
    join(Client client) throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: join: " + client);
        }

	po.joinSession(client, true);
    }


/**
 * <A NAME="SD_LISTCHANNELNAMES"></A>
 * <EM>listChannelNames</EM> list the names of the channels that are known
 * to this session.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist.
 * </PRE>
 *
 * @return an array of names of the Channels that are known to this Session.
 */

    public String[]
    listChannelNames() throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: listChannelNames.");
        }

        return(po.listChannelNames());
    }


/**
 * <A NAME="SD_CREATECHANNEL"></A>
 * <EM>createChannel</EM> creates a channel with the given name which can
 * then be used with the various channel operations. This channel is then
 * known to the session. No channel manager is associated with this channel;
 * any client which has already joined the session may freely join this channel.
 *
 * If the given name already exists (ie. a channel is already created with
 * this name), then a SharedDataException will be thrown
 *
 * @param <EM>channelName</EM> the name to give this channel. If name is null
 * then a default unique name will be used.
 * @param <EM>reliable</EM> whether the channel is reliable. In other words
 * whether data delivery is guarenteed.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist.
 *      CHANNEL_EXISTS  - this channel name already exists in this session.
 * </PRE>
 *
 * @return the name of the newly created Channel.
 */

    public String
    createChannel(String channelName,
		  boolean reliable) throws SharedDataException {

	return(createChannel(channelName, null, reliable));
    }


/**
 * <A NAME="SD_CREATECHANNELWITHMANAGER"></A>
 * <EM>createChannel</EM> creates a channel with the given name which
 * can then be used with the various channel operations. This channel is then
 * known to the session. A channel manager is associated with this channel;
 * clients are authenticated before they are allowed to join the channel
 * (ie. the equivalent of a private channel).
 *
 * @param <EM>channelName</EM> the name to give this channel. If name is null
 * then a default unique name will be used.
 * @param <EM>channelManager</EM> the manager for this channel.
 * @param <EM>reliable</EM> whether the channel is reliable. In other words
 * whether data delivery is guarenteed.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist.
 *      CHANNEL_EXISTS  - this channel name already exists in this session.
 *      MANAGER_EXISTS  - a manager already exists for this channel.
 * </PRE>
 *
 * @return the name of the newly created Channel.
 */
 
    public String
    createChannel(String channelName, ChannelManager channelManager,
		  boolean reliable) throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: createChannel:"  +
                               " name: "     + channelName +
                               " manager: "  + channelManager +
			       " reliable: " + reliable);
        }

        synchronized (noChannels) {
            int channelNo = noChannels.intValue() + 1;

	    if (channelName == null) {
                channelName = "__channel" + channelNo;
	    }
	    noChannels = new Integer(channelNo);
        }
        return(po.createChannel(channelName, channelManager, reliable));
    }


/**
 * <A NAME="SD_LISTTOKENNAMES"></A>
 * <EM>listTokenNames</EM> list the names of the tokens that are known
 * to this session.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist.
 * </PRE>
 *
 * @return an array of names of Tokens that are known to this Session.
 */

    public String[]
    listTokenNames() throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: listTokenNames.");
        }

        return(po.listTokenNames());
    }


/**
 * <A NAME="SD_CREATETOKEN"></A>
 * <EM>createToken</EM> creates a token with the given name which can
 * then be used with the various token operations. This token is then
 * known to the session. No token manager is associated with this token;
 * any client which has already joined the session may freely join this token.
 *
 * @param <EM>tokenName</EM> the name to give this token. If name is null
 * then a default unique name will be used.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist.
 *      TOKEN_EXISTS    - this token name already exists in this session.
 * </PRE>
 *
 * @return the name of the newly created Token.
 */

    public String
    createToken(String tokenName) throws SharedDataException {

        return(createToken(tokenName, null));
    }


/**
 * <A NAME="SD_CREATETOKENWITHMANAGER"></A>
 * <EM>createToken</EM> creates a token with the given name which
 * can then be used with the various token operations. This token is then
 * known to the session. A token manager is associated with this token;
 * clients are authenticated before they are allowed to join the token
 * (ie. the equivalent of a private token).
 *
 * @param <EM>tokenName</EM> the name of the token to create.
 * @param <EM>tokenManager</EM> the manager of this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist.
 *      TOKEN_EXISTS    - this token name already exists in this session.
 *      MANAGER_EXISTS  - a manager already exists for this token.
 * </PRE>
 *
 * @return the name of the newly created Token.
 */
 
    public String
    createToken(String tokenName, TokenManager tokenManager)
                                           throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: createToken:"  +
                               " name: "    + tokenName +
                               " manager: " + tokenManager);
        }

        synchronized (noTokens) {
            int tokenNo = noTokens.intValue() + 1;

            if (tokenName == null) {
                tokenName = "__token" + tokenNo;
            }
            noTokens = new Integer(tokenNo);
        }

        return(po.createToken(tokenName, tokenManager));
    }


/**
 * <A NAME="SD_ADDBYTEARRAYOBSERVER"></A>
 * <EM>addByteArrayObserver</EM> send a message to the session server to add
 * this observer to the list of observers that will be notified when the value
 * of this shared byte array, in this session changes.
 *
 * Note that if the shared byte array with this name doesn't exist, the
 * observer is notified when/if a shared byte array of that name is created.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * @param <EM>byteArrayName</EM> the name of the shared byte array to observe.
 * @param <EM>observer</EM> the Shared Byte Array observer to add.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 * </PRE>
 */

    public void
    addByteArrayObserver(String byteArrayName,
               SharedByteArrayObserver observer) throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: addByteArrayObserver:" +
                               " byte array name: " + byteArrayName +
                               " observer: "        + observer);
        }

        po.addByteArrayObserver(byteArrayName, observer);
    }


/**
 * <A NAME="SD_CREATEBYTEARRAY"></A>
 * <EM>createByteArray</EM> send a message to the session server to create a
 * new shared byte array with the given name for this session. If a shared
 * byte array of this name doesn't already exist, then it's created in the
 * server.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * The shared byte array is created locally before a message is sent to the
 * server. This is so an observer can be added can be added which can be
 * automatically informed, if this shared byte array already exists, and
 * the value for it, is different than the value being passed across in the
 * message.
 *
 * Note that if the shared byte array already existed in the server and
 * contained a different value than given here, than a change message will
 * be sent back to this session proxy indicating the real value. This in
 * turn is based on to the called of the createByteArray method.
 *
 * @param <EM>client</EM> a client successfully joined to this session.
 * @param <EM>observer</EM> the observer to associate locally with this
 * shared byte array.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to create.
 * @param <EM>value</EM> the value of the byte array.
 * @param <EM>length</EM> the length of the byte array.
 *      
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 *      NO_SUCH_CLIENT  - client given does not exist in the server.
 * </PRE>
 */     
        
    public void
    createByteArray(Client client,
		    SharedByteArrayObserver observer, String byteArrayName,
                    byte[] value, int length) throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: createByteArray:" +
                               " client name : "    + client.getName() +
			       " observer name: "   + observer.getName() +
			       " byte array name: " + byteArrayName +
			       " value: "           + value +
			       " length: "          + length);
        }

	po.createByteArray(client, observer, byteArrayName, value, length);
    }


/**
 * <A NAME="SD_DELETEBYTEARRAYOBSERVER"></A>
 * <EM>deleteByteArrayObserver</EM> send a message to the session server to 
 * delete this observer from the list of observers that will be notified when 
 * the value of this shared byte array, in this session changes.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * @param <EM>byteArrayName</EM> the name of the shared byte array to observe.
 * @param <EM>observer</EM> the Shared Byte Array observer to delete.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 *      NO_SUCH_OBSERVER - observer not observing this shared byte array.
 * </PRE>
 */

    public void
    deleteByteArrayObserver(String byteArrayName,
               SharedByteArrayObserver observer) throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: deleteByteArrayObserver:" +
                               " byte array name: " + byteArrayName +
                               " observer: "        + observer);
        }

        po.deleteByteArrayObserver(byteArrayName, observer);
    }


/**
 * <A NAME="SD_DESTROYBYTEARRAY"></A>
 * <EM>destroyByteArray</EM> send a message to the session server to destroy
 * the shared byte array with the given name for this session. 
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * All observers of this shared byte array are informed that it has been
 * destroyed.
 *
 * @param <EM>client</EM> a client successfully joined to this session.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to destroy.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 *      NO_SUCH_CLIENT  - client given does not exist in the server.
 *      NO_SUCH_VALUE   - shared byte array of this name does not exist in
 *                        the server.
 * </PRE>       
 */

    public void
    destroyByteArray(Client client, 
                     String byteArrayName) throws SharedDataException {
        if (Session_Debug) {
	    System.out.println("Session: destroyByteArray:" +
				" client name: "     + client.getName() +
				" byte array name: " + byteArrayName);
	}

        po.destroyByteArray(client, byteArrayName);
    }


/**
 * <A NAME="SD_LISTBYTEARRAYNAMES"></A>
 * <EM>listByteArrayNames</EM> list the names of the shared byte arrays that
 * are known to this session.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 * </PRE>
 *
 * @return an array of names of the shared byte arrays that are known to
 * this Session.
 */

    public String[]
    listByteArrayNames() throws SharedDataException {
        if (Session_Debug) {
            System.out.println("Session: listByteArrayNames.");
        }

        return(po.listByteArrayNames());
    }


/**
 * <A NAME="SD_UPDATEBYTEARRAY"></A>
 * <EM>updateByteArray</EM> send a message to the session server to update a
 * shared byte array with the given name for this session. The session server
 * sends a change notification to all the observers of this shared byte array.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * @param <EM>client</EM> a client successfully joined to this session.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to update.
 * @param <EM>value</EM> the new value of the byte array.
 * @param <EM>length</EM> the length of the byte array.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 *      NO_SUCH_CLIENT  - client given does not exist in the server.
 *      NO_SUCH_VALUE   - shared byte array of this name does not exist in
 *                        the server.
 * </PRE>
 */

    public void
    updateByteArray(Client client, String byteArrayName,
                    byte[] value, int length) throws SharedDataException {
        if (Session_Debug) {
	    System.out.println("Session: updateByteArray:" +
                               " client name: "     + client.getName() +
			       " byte array name: " + byteArrayName +
                               " value: "           + value +
                               " length: "          + length);
	}

	po.updateByteArray(client, byteArrayName, value, length);
    }


/**
 * <A NAME="SD_SETNAME"></A>
 * <EM>setName</EM> set the name of this session. This needs to be a separate
 * method as the session is now dynamically created with newInstance(), and
 * therefore can't be done in the constructor. It should only be called by
 * the methods in the SessionFactory class.
 *
 * @param <EM>sessionName</EM> the name of the session.
 */

    protected void
    setName(String sessionName) {
        if (Session_Debug) {
            System.out.println("Session: setName:" +
				" session name: " + sessionName);
        }
    }
}
