
/*
 * @(#)AuthenticateClient.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.StringTokenizer;

/**
 * Shared Data proxy authentication client class.
 *
 * @version     0.94 6 April 1997
 * @author      Rich Burridge
 */

class
AuthenticateClient extends socketSharedDataObject implements Client, Runnable {

    /** The client-side session associated with this authentication client. */
    private Session session;

    /** The kind of object, the client is trying to join:
     *  one of SESSION, CHANNEL or TOKEN.
     */
    String objectType = null;

    /** The name of the object, the client is trying to join. */
    String objectName = null;

    /** The id to use in client/server messages. */
    int id = 0;

    /** The name of this client. */
    private String name;

    /** The client-side proxy associated with the Session for this object. */
    private SessionProxy sp;

    /** The client-side managable object. */
    private Manageable manageable;

    /** The client-side manager associated with this manageable object. */
    private SharedDataManager manager;


/**
 * <A NAME="SD_AUTHENTICATECLIENT"></A>
 * <EM>AuthenticateClient</EM> the constructor for the AuthenticateClient
 * class.
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>session</EM> the client-side session creating this dummy
 * authentication client.
 * @param <EM>clientName</EM> the name of the client being constructed.
 * The client name cannot include '\u0001' or newline characters.
 * @param <EM>objectType</EM> the type of object the client is trying to join.
 * @param <EM>objectName</EM> the name of the object the client is trying
 * to join.
 * @param <EM>manager</EM> the client-side manager.
 * @param <EM>manageable</EM> the client-side manageable object.
 */

    AuthenticateClient(int id, Session session, String clientName,
		       String objectType, String objectName,
		       SharedDataManager manager, Manageable manageable) {
        if (AuthenticateClient_Debug) {
            System.out.println("AuthenticateClient: constructor: " +
				" id: "          + id +
				" session: "     + session +
				" client name: " + clientName +
				" object type: " + objectType +
				" object name: " + objectName +
				" manager: "     + manager +
				" manageable: "  + manageable);
        }

        this.id         = id;
	this.session    = session;
        this.name       = clientName;
	this.objectType = objectType;
	this.objectName = objectName;
	this.manager    = manager;
	this.manageable = manageable;

	sp = (SessionProxy) session.po.getProxy();
    }    


/**
 * <A NAME="SD_AUTHENTICATE"></A>
 * <EM>authenticate</EM> used to authenticate a client for potentially
 * joining a session. The Session, Channel or Token Manager will be doing
 * this client validation.
 *
 * The manager sends the proxy client a challenge. The client sends the
 * challenge (via the session-server) to the real client trying to join
 * the session/channel/token. It responds. The response is sent back (via
 * the session-server) to this proxy client, and returned to the manager.
 * It then determines whether the client should be admitted or not.
 *
 * @param <EM>challenge</EM> the challenge given by the manager to
 * the client.
 *
 * @return the response by the remote client to the managers challenge.
 */

    public String
    authenticate(String challenge) {
        String waitMessage  = addMessageHeader(session.getName(), id,
			      Client.M_Client + T_Sep + T_Authenticate +
			      T_Sep + objectType + T_Sep + objectName +
			      T_Sep + name);
        String message      = waitMessage + T_Sep + challenge;
        String reply        = null;
	String response     = null;
        int    retval       = 0;
	StringTokenizer tok = null;

        if (AuthenticateClient_Debug) {
            System.out.println("AuthenticateClient: authenticate:" +
			       " challenge: " + challenge);
        }

	reply = sp.getManagerThread().writeLine(waitMessage, message);
	tok = new StringTokenizer(reply.substring(waitMessage.length()+1),
                                  T_Sep, false);

	retval = Integer.parseInt(tok.nextToken());
	if (retval != 0) {
	    System.out.println("AuthenticateClient: authenticate:" +
                               " invalid response.");
	} else {
	    response = tok.nextToken();
	}
	return(response);
    }


/**
 * <A NAME="SD_GETNAME"></A>
 * <EM>getName</EM> get the name of this client.
 * The client name cannot include '\u0001' or newline characters. 
 *
 * @return the name of the client.
 */

    public String
    getName() {
        if (AuthenticateClient_Debug) {
            System.out.println("AuthenticateClient: getName: " + name);
        }
	return(name);
    }


/**
 * <A NAME="SD_CLIENTCHANGED"></A>
 * <EM>clientChanged</EM> is called when a client change has been observed.
 *
 * It should never be called for the proxy autheentication client.
 *
 * @param <EM>event</EM> the client changed event containing more information
 * on exactly what has changed.
 */

    public void
    clientChanged(ClientEvent event) {
        if (AuthenticateClient_Debug) {
            System.out.println("AuthenticateClient: clientChanged:" +
			       " - this should never be called.");
        }
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
        String  reply    = null;
        boolean admitted = false;

        if (AuthenticateClient_Debug) {
            System.out.println("AuthenticateClient: run.");
        }

        if (objectType.equals(Channel.M_Channel)) {
	    admitted = ((ChannelManager) manager).channelRequest(
                            (Channel) manageable, this);
        } else if (objectType.equals(Session.M_Session)) {
	    admitted = ((SessionManager) manager).sessionRequest(
                            (Session) manageable, this);
        } else if (objectType.equals(Token.M_Token)) {
	    admitted = ((TokenManager) manager).tokenRequest(
                            (Token) manageable, this);
        }                                      
                                               
        reply = addMessageHeader(session.getName(), id,
                    Client.M_Client + T_Sep + T_Join + T_Sep + objectType +
                    T_Sep + name + T_Sep + admitted);

        sp.proxyThread.writeLine(null, reply);
    }
}
