
/*
 * @(#)UDPChannelProxyThread.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.util.StringTokenizer;

/** 
 * Shared Data UDP Channel proxy thread class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */
 
class
UDPChannelProxyThread extends UDPSocketThread {

    /** The client-side session associated with this channel thread. */
    private Session session;

    /** The client-side channel associated with this channel thread. */
    private Channel channel;

/**
 * <A NAME="SD_UDPCHANNELPROXYTHREAD"></A>
 * <EM>UDPChannelProxyThread</EM> create a thread (and therefore an associated
 * separate socket), to handle data sent over the channel at a specific
 * priority.
 *
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>channel</EM> the channel in question.
 * @param <EM>host</EM> the server host name.
 * @param <EM>port</EM> the port number on the server to connect to.
 */

    public
    UDPChannelProxyThread(Session session, Channel channel,
			  String host, int port) {
        super(host, port);

        if (UDPChannelProxyThread_Debug) {
            System.out.println("UDPChannelProxyThread: constructor:" +
                               " session: " + session +
                               " channel: " + channel +
                               " host: "    + host +
			       " port: "    + port);
        }
 
        this.session = session;
        this.channel = channel;
    }


/**
 * <A NAME="SD_MESSAGECONTAINSDATA"></A>
 * <EM>messageContainsData</EM> checks if this is one of the message types
 * that contains data.
 *
 * @param <EM>type</EM>   - the type of this message
 * @param <EM>action</EM> - the action for this message.
 */

    private boolean
    messageContainsData(String type, String action) {
        if (SessionProxyThread_Debug) {
            System.out.println("SessionProxyThread: messageContainsData: " +
                               " type: "   + type +
                               " action: " + action);
        }

        return(type.equals(Channel.M_Channel) &&
	       action.equals(T_DataReceived));
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>message</EM>
 */

    protected void
    handleMessage(String message) {
        StringTokenizer tok = new StringTokenizer(message, T_Sep, false);

        if (UDPChannelProxyThread_Debug) {
            System.out.println("UDPChannelProxyThread: handleMessage:" +
                               " message: " + message);
        }

        if (validMessageHeader(tok) != null) {
            ChannelProxyMessage cpm = new ChannelProxyMessage(session,
                                                              channel, null);
            cpm.handleMessage(tok);
        }
    }


/**
 * <A NAME="SD_HANDLEMESSAGEANDDATA"></A>
 * <EM>handleMessageAndData</EM>
 *
 * @param <EM>message</EM>
 * @param <EM>data</EM>
 */

    protected void
    handleMessageAndData(String message, byte data[]) {
        StringTokenizer tok = new StringTokenizer(message, T_Sep, false);

        if (UDPChannelProxyThread_Debug) {
            System.out.println("UDPChannelProxyThread: handleMessageAndData:" +
                               " message: " + message +
                               " data: "    + data);
        }

        if (validMessageHeader(tok) != null) {
            ChannelProxyMessage cpm = new ChannelProxyMessage(session,
                                                              channel, data);
         
            cpm.handleMessage(tok);
        }
    }


/**
 * <A NAME="SD__WRITE"></A>
 * <EM>_write</EM> write a message (possibly with data) over the UDP channel.
 *
 * @param <EM>waitCommand</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 * @param <EM>dataBytes</EM> the data (or null if there isn't any).
 * @param <EM>length</EM> the length (in bytes) of the data.
 *
 * @return the reply (if we were waiting for it), otherwise null.
 */

    synchronized String
    _write(String waitCommand, String message, byte dataBytes[], int length) {
	ByteArrayOutputStream ba  = new ByteArrayOutputStream();
	DataOutputStream      dos = new DataOutputStream(ba);
        DatagramPacket        dp  = null;
        String                s   = null;

        if (UDPChannelProxyThread_Debug) {
            System.out.println("UDPChannelProxyThread: _write:" +
                                " waitCommand: " + waitCommand +
                                " message: "     + message   +
                                " data: "        + dataBytes +
                                " length: "      + length);
        }

        try {
            if (waitCommand != null) {
                while (isWaiting == true) {
                    wait();
                }
                this.waitCommand = waitCommand;
                isWaiting        = true;
            }


	    dos.writeInt(length);
	    dos.writeUTF(message);
	    if (length != 0) {
		dos.write(dataBytes, 0, length);
	    }
	    dp = new DatagramPacket(ba.toByteArray(), ba.size(),
				    address, port);
            dsock.send(dp);

            while (waitCommand != null && isWaiting == true) {
                s = getMessage();

                if (UDPChannelProxyThread_Debug) {
                    System.out.println("UDPChannelProxyThread: _write:" +
                                       " about to handle:" + s);
                }

                if (s.startsWith(this.waitCommand)) {
                    if (UDPChannelProxyThread_Debug) {
                        System.out.println("UDPChannelProxyThread: _write: " +
                                           "found: " + this.waitCommand);
		    }
                    isWaiting = false;
                } else {
                    StringTokenizer tok = new StringTokenizer(s, T_Sep, false);

                    if (validMessageHeader(tok) != null) {
                        int    id     = Integer.parseInt(tok.nextToken());
			String type   = tok.nextToken();
                        String action = tok.nextToken();

                        if (messageContainsData(type, action)) {
                            handleMessageAndData(s, getData());
                        } else {
                            handleMessage(s);
                        }
                    }    
                }
            }    
        } catch (Exception e) {
            System.out.println("UDPChannelProxyThread: _write: " +
                               "exception " + e);
            if (UDPChannelProxyThread_Debug) {
                e.printStackTrace();
            }
        }
        return(s);
    }


/**
 * <A NAME="SD_PUTMESSAGEANDDATA"></A>
 * <EM>putMessageAndData</EM>
 *
 * @param <EM>s</EM>
 *
 * @exception InterruptedException
 */

    protected void
    putMessageAndData(String s) throws InterruptedException {
        StringTokenizer tok = new StringTokenizer(s, T_Sep, false);

        if (UDPChannelProxyThread_Debug) {
            System.out.println("UDPChannelProxyThread: putMessageAndData:" +
                               " message: " + s);
        }

        if (validMessageHeader(tok) != null) {

/* Handles the following message:
 *
 * CHANNEL DATARECEIVED <channel> <sender> <receiver> <priority> <length>
 *      
 * Extracts the data length from the message, then read that amount of bytes
 * and generates a second message to put in the message queue.
 */     
        
            String id     = tok.nextToken();       /* Strip off message id. */
	    String type   = tok.nextToken();       /* Strip off CHANNEL. */
            String action = tok.nextToken();       /* Get DATARECEIVED. */
        
            if (action.equals(T_DataReceived)) {
                for (int i = 0; i < 4; i++) {
                    tok.nextToken();
                }
            } else {
                System.out.println("UDPChannelProxyThread: putMessageAndData:" +
                                   " unexpected message action: " + action);
            }
        
            putMessage(s);
            putMessage(new String(data, 0));
        }
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
        StringTokenizer tok = null;

        if (UDPChannelProxyThread_Debug) {
            System.out.println("UDPChannelProxyThread: run.");
        }

        try {
            while (true) {
                String s = null;

                if ((s = getSocketMessage()) == null) {
                    break;
                }

                if (UDPChannelProxyThread_Debug) {
                    System.out.println("UDPChannelProxyThread: run:" +
                        " - received: " + s + " (" + s.length() + " bytes)");
                }

                tok = new StringTokenizer(s, T_Sep, false);
                if (validMessageHeader(tok) != null) {
                    int    id     = Integer.parseInt(tok.nextToken());
		    String type   = tok.nextToken();
                    String action = tok.nextToken();

                    if (messageContainsData(type, action)) {
                        putMessageAndData(s);
                    } else {
                        putMessage(s);
                    }
                }
 
                if (isWaiting == false) {
                    while (messages.size() > 0) {
                        s = getMessage();
 
                        if (UDPChannelProxyThread_Debug) {
                            System.out.println("UDPChannelProxyThread: run:" +
                                               " about to handle:" + s);
                        }
 
                        tok = new StringTokenizer(s, T_Sep, false);
                        if (validMessageHeader(tok) != null) {
                            int    id     = Integer.parseInt(tok.nextToken());
			    String type   = tok.nextToken();
                            String action = tok.nextToken();

                            if (messageContainsData(type, action)) {
                                handleMessageAndData(s, getData());
                            } else {
                                handleMessage(s);
                            }
                        }
                    }    
                }
            }    
        } catch (Exception e) {
            System.out.println("UDPChannelProxyThread: run - exception " + e);
            if (UDPChannelProxyThread_Debug) {
                e.printStackTrace();
            }
        }
    }
}
