// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ArcPerspective.java
// Classes: ArcPerspective
// Original Author: jrobbins@ics.uci.edu
// $Id: ArcPerspective.java,v 1.12 1997/06/10 23:42:31 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;
import java.io.*;
import uci.ui.IProps;
import uci.util.*;

/** A DiagramElement that draws arcs between ports.
 *  <A HREF="../features.html#graph_visualization_arcs">
 *  <TT>FEATURE: graph_visualization_arcs</TT></A>
 *  <A HREF="../bugs.html#arc_translate">
 *  <FONT COLOR=660000><B>BUG: arc_translate</B></FONT></A>
 */

public abstract class ArcPerspective extends DiagramElement {

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** Fig presenting the NetArc's from NetPort . */
  protected Fig _sourcePortFig;
  /** Fig presenting the NetArc's to NetPort. */
  protected Fig _destPortFig;
  /** Perspective presenting the NetArc's from NetPort's parent NetNode. */
  protected Perspective _sourcePerspective;
  /** Perspective presenting the NetArc's to NetPort's parent NetNode. */
  protected Perspective _destPerspective;
  /** Fig that presents the NetArc. */
  protected Fig _arcFig;

  ////////////////////////////////////////////////////////////////
  // constructors

  public ArcPerspective(Fig s, Fig d,
			Perspective sp, Perspective dp,
			NetArc a) {
    _sourcePortFig = s;
    _destPortFig = d;
    _sourcePerspective = sp;
    _destPerspective = dp;
    Rectangle bbox = getBoundingBox();
    owner(a);
    _arcFig = makeArcFig();
  }

  public ArcPerspective() { _arcFig = makeArcFig(); }

  protected abstract Fig makeArcFig();

  ////////////////////////////////////////////////////////////////
  // accessors

  public void sourcePortFig(Fig fig) { _sourcePortFig = fig; }

  public void destPortFig(Fig fig) { _destPortFig = fig; }

  public void sourcePerspective(Perspective pers) {_sourcePerspective = pers; }

  public void destPerspective(Perspective pers) { _destPerspective = pers; }

  public void owner(Object own) {
    Object oldOwner = owner();
    if (oldOwner != null && oldOwner instanceof Observable) {
      ((Observable)oldOwner).deleteObserver(this);
    }
    if (own instanceof Observable) {
      ((NetArc)own).addPersistantObserver(this);
    }
    super.owner(own);
  }

    public Object get(String key) {
    if (owner() instanceof IProps) {
      IProps node = (IProps) owner();
      Object res = node.get(key);
      if (res != null) return res;
    }
    return super.get(key);
  }

  public boolean put(String key, Object value) {
    boolean res = false;
    if (owner() instanceof IProps) {
      IProps node = (IProps) owner();
      if (node != null) res = node.put(key, value);
    }
    return res || super.put(key, value);
  }

  public Enumeration keysIn(String cat) {
    if (owner() instanceof IProps) {
      IProps node = (IProps) owner();
      if (node != null)
	return new EnumerationComposite(node.keysIn(cat), super.keysIn(cat));
    }
    return super.keysIn(cat);
  }

  public boolean canPut(String key) {
    if (owner() instanceof IProps) {
      IProps node = (IProps) owner();
      if (node != null) return node.canPut(key);
    }
    return super.canPut(key);
  }

  ////////////////////////////////////////////////////////////////
  // Routing related methods

  protected void computeRoute() { }

  ////////////////////////////////////////////////////////////////
  // DiagramElement API

  /** Draw this object when the user has selected it. */
  public void drawSelected(Graphics g) { _arcFig.drawSelected(g); }

  /** Reply the bounding box for this arc. */
  public Rectangle getBoundingBox() { return _arcFig.getBoundingBox(); }

  /** Reply true iff this arc contains the given point.
   * @see DiagramElement#inside */
  public boolean inside(int x, int y) { return _arcFig.inside(x, y); }

  /** Reply true iff this arc intersects the given rectangle. */
  public boolean intersects(Rectangle r) { return _arcFig.intersects(r); }

  public void dispose(Editor ed) {
    System.out.println("disposing: " + toString());
    ((NetArc)owner()).dispose();
    super.dispose(ed);
  }

  public void removeFrom(Editor ed) {
    super.removeFrom(ed);
    owner(null);
  }

  ////////////////////////////////////////////////////////////////
  // drawing methods

  /** Draw this object. */
  public void draw(Graphics g) {
    computeRoute();
    g.setColor(Color.black);
    drawArc(g);
  }

  /** Draw the arc. */
  protected void drawArc(Graphics g) { _arcFig.draw(g); }

  ////////////////////////////////////////////////////////////////
  // notifications and updates

  /** Reply a new selection object that can be used to manipulate this
   *  object.  For now, use SelectionHandles.  Needs-More-Work: Users
   *  should be able to adjust arcs by dragging on handles. */
  public Selection selectionObject() {
    return new SelectionHandles(this);
  }

  public void update(Observable o, Object arg) {
    if (arg instanceof Vector) {
      Vector v = (Vector) arg;
      String s = (String) v.elementAt(0);
      Object obj = v.elementAt(1);
      if (s.equals(Globals.REMOVE) && obj == owner()) {
	removeFrom(null);
      }
    }
  }

} /* end class ArcPerspective */
