// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ArcPerzRectiline.java
// Classes: ArcPerzRectiline
// Original Author: Jason Robbins
// $Id: ArcPerzRectiline.java,v 1.7 1997/06/10 23:42:33 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;
import java.io.*;

/** A DiagramElement that draws arcs between ports. This version
 *  automatically routes a rectilinear arc. The routing is not very
 *  good. It avoids the source and sink nodes and no other nodes. It is
 *  basically case-analysis, and some of the cases are wrong or
 *  missing. In any case, the user can edit the arc by dragging
 *  handles. The 0th and last handles are fixed in position so that
 *  they stay connected to ports. If the user drags a handle next to a
 *  fixed handle, a new vertex is automatically inserted.
 *
 * @see FigPoly */

public class ArcPerzRectiline extends ArcPerspective {

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** True if the arc has been laid out automatically once. It will
   *  not be done automatically again since the user may have edited the
   *  arc and I dont want to undo that work. */
  protected boolean _initiallyLaidOut = false;

  ////////////////////////////////////////////////////////////////
  // ArcPerspective API

  /** Instanciate a FigPoly with its rectilinear flag set. */
  protected Fig makeArcFig() {
    FigPoly res = new FigPoly(Color.black);
    res.rectilinear(true);
    res.fixedHandles(1);
    res.setFilled(false);
    return res;
  }

  ////////////////////////////////////////////////////////////////
  // event handlers

  /** Pass keystorkes to the FigPoly, so that the user can insert and
   *  remove vertices. */
  public boolean keyDown(Event e, int key) {
    return _arcFig.keyDown(e, key) || super.keyDown(e, key);
  }

  /** Pass this through to the FigPoly. */
  public void insertAfterSelectedVertex() {
    ((FigPoly)_arcFig).insertAfterSelectedVertex();
  }

  /** Pass this through to the FigPoly. */
  public void removeSelectedVertex() {
    ((FigPoly)_arcFig).removeSelectedVertex();
  }

  /** Pass this through to the FigPoly. */
  public int pickHandle(int x, int y) { return _arcFig.pickHandle(x, y); }

  /** Pass this through to the FigPoly. */
  public void dragHandle(int mx, int my, int an_x, int an_y, Handle h) {
    if (h.index > 0 && h.index < ((FigPoly) _arcFig).npoints() -1)
      _arcFig.dragHandle(mx, my, an_x, an_y, h);
    else
      System.out.println("dragging on an end point!");
  }

  ////////////////////////////////////////////////////////////////
  // routing methods

  /** Find the route that the arc should follow.  Basically case
   *  analysis to route around source and destination nodes.
   *  Needs-More-Work: A better algorithm would really be useful.
   *  Needs-More-Work: Sometimes the arc can get non-rectilinear. */
  protected void computeRoute() {
    if (!_initiallyLaidOut) {
      layoutArc();
      _initiallyLaidOut = true;
    }
    Point srcPt = _sourcePortFig.center();
    Point dstPt = _destPortFig.center();
    FigPoly p = ((FigPoly) _arcFig);
    p.setEndPoints(srcPt, dstPt);
  } /* end computeRoute */

  /** Internal function to actually compute the layout of the line if
   *  it has never been done on that line before.
   *  <A HREF="../bugs.html#rectilinear_arc_constraints">
   *  <FONT COLOR=660000><B>BUG: rectilinear_arc_constraints</B></FONT></A>
   *  <A HREF="../bugs.html#rectilinear_arc_points">
   *  <FONT COLOR=660000><B>BUG: rectilinear_arc_points</B></FONT></A>
   */
  protected void layoutArc() {
    int npoints = 0;
    int xpoints[] = new int[16];
    int ypoints[] = new int[16];
    Point srcPt = _sourcePortFig.connectionPoint(_destPortFig.center());
    Point dstPt = _destPortFig.connectionPoint(_sourcePortFig.center());
    Rectangle srcRR = _sourcePerspective.routingRect();
    Rectangle dstRR = _destPerspective.routingRect();

    NetPort srcPort = (NetPort) _sourcePortFig.owner();
    NetPort dstPort = (NetPort) _destPortFig.owner();
    int srcSector = srcPort.getPortSector(_sourcePerspective);
    int dstSector = dstPort.getPortSector(_destPerspective);

    // first decide what layout case we have
    Point srcRRPt = routingRectPoint(srcPt, srcRR, srcSector);
    Point dstRRPt = routingRectPoint(dstPt, dstRR, dstSector);

    if (srcSector == 1 || srcSector == -1) {
      xpoints[npoints] = srcPt.x; ypoints[npoints++] = srcPt.y; }
    xpoints[npoints] = srcPt.x; ypoints[npoints++] = srcPt.y;
    xpoints[npoints] = srcRRPt.x; ypoints[npoints++] = srcRRPt.y;

    npoints = npoints + tryRoute(dstRRPt.x, dstRRPt.y, npoints,
				 xpoints, ypoints,
				 srcRR, dstRR, srcSector, dstSector);

    xpoints[npoints] = dstRRPt.x; ypoints[npoints++] = dstRRPt.y;
    xpoints[npoints] = dstPt.x; ypoints[npoints++] = dstPt.y;
    //   if (dstSector == 1 || dstSector == -1) {
    // xpoints[npoints] = dstPt.x; ypoints[npoints++] = dstPt.y; }
    Polygon routePoly = new Polygon(xpoints, ypoints, npoints);
    ((FigPoly)_arcFig).polygon(routePoly);
  }

  /** Reply a point on the given routing rect that is "straight out"
   *  from the connection point in the proper direction. */
  protected Point routingRectPoint(Point p, Rectangle r, int sector) {
    switch (sector) {
    case -1: return new Point(p.x, r.y);
    case 2: return new Point(r.x, p.y);
    case 1: return new Point(p.x, r.y + r.height);
    case -2: return new Point(r.x + r.width, p.y);
    default: System.out.println("error, undefined sector!");
      return p;
    }
  }

  /** Try to find a route from the last point in (xs, ys) to point (x,
   *  y).  Try to avoid the given rectangles along the
   *  way. Needs-More-Work: should allow a vector of rectangles to
   *  avoid. */
  protected int tryRoute(int x, int y, int np, int xs[], int ys[],
			 Rectangle avoid1, Rectangle avoid2,
			 int srcSector, int dstSector) {
    if (np > 12 ) return 0;
    int fx = xs[np-1], fy = ys[np-1];
    if ((fx == x || fy == y) && segOK(fx, fy, x, y, avoid1, avoid2)) {
      xs[np] = x; ys[np++] = y;
      xs[np] = x; ys[np++] = y;
      return 1;
    }
    else {
      if (segOK(fx, fy, fx, y, avoid1, avoid2) &&
	  segOK(fx, y, x, y, avoid1, avoid2)) {
	xs[np] = fx; ys[np++] = y;
	xs[np] = x; ys[np++] = y;
	return 2;
      }
      else if (segOK(fx, fy, x, fy, avoid1, avoid2) &&
	       segOK(x, fy, x, y, avoid1, avoid2)) {
	xs[np] = x; ys[np++] = fy;
	xs[np] = x; ys[np++] = y;
	return 2;
      }
      else {
	Point avoidPt = findAvoidPt(fx, fy, x, y, avoid1, avoid2);
	if ((srcSector == 1 || srcSector == -1 || fy == avoidPt.y) &&
	    fx != avoidPt.x ) {
	  xs[np] = avoidPt.x;
	  ys[np++] = fy;
	  return tryRoute(x, y, np, xs, ys, avoid1, avoid2,
			  srcSector, dstSector) + 1;
	}
	if ((srcSector == 2 || srcSector == -2 || fx == avoidPt.x) &&
	    fy != avoidPt.y) {
	  xs[np] = fx;
	  ys[np++] = avoidPt.y;
	  return tryRoute(x, y, np, xs, ys, avoid1, avoid2,
			  srcSector, dstSector) + 1;
	}
	return 0;
      }
    }
  }

  /** Find a point on the corner of one of the avoid rectangles that
   *  is a good intermediate point for the arc layout. */
  protected Point findAvoidPt(int fx, int fy, int x, int y,
			      Rectangle avoid1, Rectangle avoid2) {
    Point res = new Point(fx, fy);
    if (avoid1.x + avoid1.width < avoid2.x)
      res.x = avoid2.x;
    else if (avoid2.x + avoid2.width < avoid1.x)
      res.x = avoid1.x;
    else res.x = Math.min(avoid1.x, avoid2.x);

    if (avoid1.y + avoid1.height < avoid2.y)
      res.y = avoid2.y;
    else if (avoid2.y + avoid2.height < avoid1.y)
      res.y = avoid1.y;
    else res.y = Math.min(avoid1.y, avoid2.y);

    return res;
  }

  /** Reply true if the line segment from (x1, y1) to (x2, y2) does
   *  not intersect the given avoid rectangles. */
  protected boolean segOK(int x1, int y1, int x2, int y2,
			  Rectangle avoid1, Rectangle avoid2) {
    int xmin = Math.min(x1, x2);
    int xmax = Math.max(x1, x2);
    int ymin = Math.min(y1, y2);
    int ymax = Math.max(y1, y2);
    int rright = avoid1.x + avoid1.width;
    int rbot = avoid1.y + avoid1.height;
    if (x1 == x2) { // vertical
      if (x1 > avoid1.x && x1 < rright &&
	  ((ymin < avoid1.y && ymax > avoid1.y) ||
	   (ymin < rbot && ymax > rbot)))
	return false;
    }
    if (y1 == y2) { // horizontal
      if (y1 > avoid1.y && y1 < rbot &&
	  ((xmin < avoid1.x && xmax > avoid1.x) ||
	   (xmin < rright && xmax > rright)))
	return false;
    }
    rright = avoid2.x + avoid2.width;
    rbot = avoid2.y + avoid2.height;
    if (x1 == x2) { // vertical
      if (x1 > avoid2.x && x1 < rright &&
	  ((ymin < avoid2.y && ymax > avoid2.y) ||
	   (ymin < rbot && ymax > rbot)))
	return false;
    }
    if (y1 == y2) { // horizontal
      if (y1 > avoid2.y && y1 < rbot &&
	  ((xmin < avoid2.x && xmax > avoid2.x) ||
	   (xmin < rright && xmax > rright)))
	return false;
    }
    return true;
  }

} /* end class ArcPerzRectiline */

