// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: PropSheetCategory.java
// Interfaces: PropSheetCategory
// Original Author: jrobbins@ics.uci.edu
// $Id: PropSheetCategory.java,v 1.6 1997/06/10 23:39:12 jrobbins Exp $

package uci.ui;
import java.awt.*;
import java.util.*;
//import symantec.itools.awt.NumericSpinner;

/** A PropSheet that shows only properties in a given Category. The
 *  display of the properties is in a fairly random order down the
 *  sheet.  This class also has several static registration methods
 *  that control how specific properties will be displayed and
 *  editied. */

public class PropSheetCategory extends PropSheet {

  ////////////////////////////////////////////////////////////////
  // constants
  public static final String dots = ".  .  .  .  .  .  .  .  .  .  .  .  .  .";

  public static final int MIN_UPDATE = 200;

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** Given a property name, which AWT component edits its value. */
  protected Hashtable _keysComps;
  /** Given an AWT component, what is the name of the property it edits. */
  protected Hashtable _compsKeys;
  protected Hashtable _inUse;
  protected Hashtable _labels;
  protected Hashtable _shown;
  protected String _category = "Misc";
  protected static Hashtable _enumProps = new Hashtable(30);
  protected static Hashtable _bigTextProps = new Hashtable(30);
  protected long _lastUpdateTime = System.currentTimeMillis();


  ////////////////////////////////////////////////////////////////
  // constructors

  public PropSheetCategory() {
    super();
    setTabName("no key");
    //{{INIT_CONTROLS
    setLayout(new PropsGridLayout(4, 4));
    //setLayout(new GridLayout(0, 2, 4, 4));
    //setLayout(new FlowLayout());
    //addNotify();
    resize(insets().left + insets().right + 250,insets().top + insets().bottom + 300);
    setFont(getPropertiesFont());
    setBackground(new Color(12632256));
    //}}
    _keysComps = new Hashtable(20);
    _compsKeys = new Hashtable(20);
    _labels = new Hashtable(20);
    _inUse = new Hashtable(20);
    _shown = new Hashtable(20);
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  public String getCategory() { return _category; }

  public void setCategory(String cat) {
    if (cat == _category) return;
    _category = cat;
    setTabName(cat);
    if (_sel != null) {
      updateComponents();
      repaint();
    }
  }

  public Component addKeyComp(String key) {
    Component comp = (Component) _keysComps.get(key);
    if (comp == null || !rightType(key, comp)) {
      comp = makeComp(key);
      _keysComps.put(key, comp);
      _compsKeys.put(comp, key);
      Label lab = new Label(key + dots);
      _labels.put(key, lab);
    }
    _inUse.put(key, key);
    return comp;
  }


  public void updateKeysComps() {
    _inUse.clear();
    if (_sel == null) return;
    Enumeration keyEnum = _sel.keysIn(_category);
    while (keyEnum.hasMoreElements()) {
      String k = (String) keyEnum.nextElement();
      Component comp = addKeyComp(k);
      boolean canPut = _sel.canPut(k);
      if (comp instanceof TextComponent)
	((TextComponent)comp).setEditable(canPut);
      else comp.enable(canPut);
    }
  }

  public void show(String key) {
    Label lab = (Label) _labels.get(key);
    Component comp = (Component) _keysComps.get(key);
    setComponentValue(key, comp);
    if (_shown.containsKey(key)) return;
    add(lab);
    add(comp);
    lab.show();
    comp.show();
    _shown.put(key, key);
  }

  public void hide(String key) {
    if (!_shown.containsKey(key)) return;
    Label lab = (Label)_labels.get(key);
    Component comp = (Component)_keysComps.get(key);
    lab.hide();
    comp.hide();
    remove(lab);
    remove(comp);
    _shown.remove(key);
  }

  /** Return true if the component I used the last time I edited this
   * property is still suitable for the current value of this
   * property.  This PropSheet works faster when this method returns
   * true a lot. The basic assumption is that different objects use
   * values of the same type for a given property. */
  public boolean rightType(String key, Component comp) {
    Object value =_sel.get(key);
    if (value == null) return !(comp instanceof Label);
    else if (value instanceof String)
      return (comp instanceof Label) ||
	(comp instanceof Choice) ||
	(comp instanceof TextComponent);
    else if (value instanceof Integer)
      //      return (comp instanceof NumericSpinner);
      return (comp instanceof TextComponent) ||
	(comp instanceof Choice);
    else if (value instanceof Boolean)
      return (comp instanceof Checkbox);
    else if (value instanceof Color)
      return (comp instanceof PropEditorColor);
    else if (value instanceof Rectangle)
      return (comp instanceof PropEditorRect);
    else return (comp instanceof Label);
  }

  /** If I don't have a comp for a given property name, or the last
   * comp is not suitable, then make a new AWT component (possibly a
   * PropertyEditor). */
  public Component makeComp(String key) {
    Object value =_sel.get(key);
    if (value == null) return new Label(key + " is null");
    Component comp;
    if (_enumProps.containsKey(key)) {
      Choice c = new Choice();
      Vector items = (Vector)_enumProps.get(key);
      Enumeration itemEnum = items.elements();
      while (itemEnum.hasMoreElements()) {
	Object item = itemEnum.nextElement();
	c.addItem(item.toString());
      }
      c.select(0);
      comp = c;
    }
    else if (_bigTextProps.containsKey(key))
      comp = new TextArea((String) value,
			  ((Integer)_bigTextProps.get(key)).intValue(),
			  30);
    else if (value instanceof String)
      comp = new TextField((String) value, 30);

    else if (value instanceof Integer) {
      //       NumericSpinner ns = new NumericSpinner();
      //       ns.setCurrent(((Integer) value).intValue());
      //       ns.setMin(Integer.MIN_VALUE);
      //       ns.setMax(Integer.MAX_VALUE);
      //       comp = ns;
      return (new TextField(value.toString(), 10));
    }
    else if (value instanceof Boolean) {
      comp = new Checkbox(key);
      ((Checkbox)comp).setState(value.equals(Boolean.TRUE));
    }
    else if (value instanceof Color)
      comp = new PropEditorColor((Color) value);
    else if (value instanceof Rectangle)
      comp = new PropEditorRect((Rectangle) value);
    else comp = new Label(value.toString());
    return comp;
  }

  /** Display the value of a given property */
  public void setComponentValue(String key, Component comp) {
    Object value = _sel.get(key);
    if (value == null) return;
    if (value instanceof String && comp instanceof TextComponent)
      ((TextComponent)comp).setText((String) value);
    else if (comp instanceof Choice) {
      Vector items = (Vector) _enumProps.get(key);
      if (items.contains(value))
	((Choice)comp).select(items.indexOf(value));
    }
//     else if (value instanceof Integer && comp instanceof NumericSpinner) {
//       NumericSpinner ns = (NumericSpinner) comp;
//       ns.setCurrent(((Integer) value).intValue());
//     }
    else if (value instanceof Integer && comp instanceof TextComponent) {
      TextComponent tc = (TextComponent) comp;
      tc.setText(((Integer) value).toString());
    }
    else if (value instanceof Boolean && comp instanceof Checkbox) {
      ((Checkbox)comp).setState(value.equals(Boolean.TRUE));
    }
    else if (value instanceof Color && comp instanceof PropEditorColor)
      ((PropEditorColor)comp).setColor((Color) value);
    else if (value instanceof Rectangle && comp instanceof PropEditorRect)
      ((PropEditorRect)comp).setRectangle((Rectangle) value);
    else if (comp instanceof Label)
      ((Label)comp).setText(value.toString());
  }

  ////////////////////////////////////////////////////////////////
  // event handling

  public boolean lostFocus(Event event, Object what) {
    Component comp = (Component) event.target;
    String key = (String) _compsKeys.get(comp);
    if (key == null) return super.action(event, what);
    Object value = null;
    if (comp instanceof TextComponent) {
      value = ((TextComponent)comp).getText();
      if (_sel != null && _sel.get(key) instanceof Integer) {
	try { value = Integer.valueOf((String)value); }
	catch (java.lang.NumberFormatException ignore) { }
      }
    }
    store(key, value);
    return true;
  }

  public boolean action(Event event, Object what) {
    Component comp = (Component) event.target;
    String key = (String) _compsKeys.get(comp);
    if (key == null) return super.action(event, what);
    Object value = null;
    if (comp instanceof TextComponent) {
      value = ((TextComponent)comp).getText();
      if (_sel.get(key) instanceof Integer) {
	try { value = Integer.valueOf((String)value); }
	catch (java.lang.NumberFormatException ignore) { }
      }
    }
    else if (comp instanceof Choice) {
      Vector items = (Vector) _enumProps.get(key);
      int index = ((Choice)comp).getSelectedIndex();
      value = items.elementAt(index);
    }
    else if (comp instanceof Checkbox)
      value = ((Checkbox)comp).getState() ? Boolean.TRUE : Boolean.FALSE;
    else if (comp instanceof PropEditorColor)
      value = ((PropEditorColor)comp).getColor();
    else if (comp instanceof PropEditorRect)
      value = ((PropEditorRect)comp).getRectangle();
//     else if (comp instanceof NumericSpinner)
//       value = new Integer(((NumericSpinner)comp).getCurrent());
    store(key, value);
    return true;
  }

  ////////////////////////////////////////////////////////////////
  // update methods

  public void addNotify() {
    if (_keysComps != null)
      super.addNotify();
  }

  public boolean canEdit(IProps item) {
    return item != null &&
      item.keysIn(_category).hasMoreElements() &&
      super.canEdit(item);
  }

  public void updateComponents() {
    super.updateComponents();
    updateKeysComps();
    Enumeration keysEnum = _keysComps.keys();
    while (keysEnum.hasMoreElements()) {
      String k = (String) keysEnum.nextElement();
      if (_inUse.containsKey(k)) show(k); else hide(k);
    }
    validate();
  }

  public void updateComponent(String key) {
    // special case for bounding box, because too many updates make dragging
    // choppy
    if (key.equals("Bbox") &&
	_lastUpdateTime + MIN_UPDATE > System.currentTimeMillis())
      return;
    Component comp = (Component) _keysComps.get(key);
    if (comp != null) setComponentValue(key, comp);
    _lastUpdateTime = System.currentTimeMillis();
  }

  ////////////////////////////////////////////////////////////////
  // registration of properties for special handleing

  public static void addEnumProp(String key, Vector items) {
    _enumProps.put(key, items);
  }
  public static void addBigTextProp(String key, int nLines) {
    _bigTextProps.put(key, new Integer(nLines));
  }


} /* end class PropSheetCategory */
