package webflow.backend;

import java.util.*;
import java.net.*;
import javax.servlet.*;
import java.io.*;

/**
 * ConnectionManager is a jeeves Servlet. It maintains a list of all ports
 * that exist under one ModuleManager, and establishes connections between 
 * ports, via communication with another ConnectionManager
 */
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class ConnectionManager extends GenericServlet implements Runnable {
  // Hashtable of PortRepresentations, indexed by the IDs of the ports 
  // they represent. 
  static Hashtable portList = null;

  // Server socket on which the ConnectionManager will listen for incoming 
  // requests
  ServerSocket clientRequests; 
  ServerSocket portConnections;

  // host name, and port number on which the server socket is connected
  // there are two server sockets: one on which requests from the clients
  // are received, and another on which connections from remote ports 
  // are received. 
  String hostName; 
  int portNumberRequests; 
  int portNumberConnections;

  // thread which will service the clientRequests socket
  Thread socketThread;

  // factory to make IDs. Must be static, as it may be called by registerPort
  static IDFactory factory = null;

  // diagnostic object
  Diagnostic diag;

  // diagnostics on or off (true = on, false = off)
  static final boolean DIAGNOSTICS_ACTIVE = true;

  /**
   * initialize the ConnectionManager servlet. This method is called when the 
   * servlet will be calle for the first time. It will initialize the hash
   * table for the ports and will initialize the socket, with wich the 
   * ConnectionManager will communicate.
   */
  public void init(ServletConfig params){
    System.out.println("in CM");

    try
    {  super.init(params);
    } catch (ServletException e)
      {  System.out.println(" exception in init() of CM ");
         e.printStackTrace();
      }

    clientRequests = null;
    portConnections = null;
    portNumberRequests = 0;
    portNumberConnections = 0;
    hostName = "0";
    socketThread = null;

    // initialize the diagnostic object
    diag = new Diagnostic("cm.diag");
    diag.activate(DIAGNOSTICS_ACTIVE);
    diag.write("Activated diagnostics object");

    try {
	// get local host name
	InetAddress local = java.net.InetAddress.getLocalHost();
	hostName = local.getHostName();

	// open new server socket for the clients
	clientRequests = new ServerSocket(0);
	portNumberRequests = clientRequests.getLocalPort();

	// open new server socket for the connections
	portConnections = new ServerSocket(0);
	portNumberConnections = portConnections.getLocalPort();

	// finally, branch new thread, set it up as a deamon (to terminate
	// upon completion of master program), and start it up
	socketThread = new Thread(this);
	socketThread.setDaemon(true);
	socketThread.start();
    }
    catch(UnknownHostException uhe) {
    }
    catch(IOException e) {
    }
  }


  /**
   * service method is called when the servlet is refered by URL. It defines
   * the response of the servlet, wich is the address of the socket on wich 
   * this servlet will communicate with the rest of the world.
   * @param req: a ServletRequest wich contains the parametars of the request
   * @param res: a ServletResponse, an object that contains the response of the
   *             servlet.
   */
  public void service(ServletRequest req, ServletResponse res)
       throws ServletException, IOException
  {
    // one day there will be multiple requests. For now, the only request
    // is get address of ModuleManager's socket. 
    res.setContentType("text/plain");
    OutputStream os = res.getOutputStream();
    PrintStream out = new PrintStream( os );
    out.println(hostName);
    out.println(portNumberRequests);
    out.flush();
  }


  /**
   * Provides an information about the servlet.
   * @return String: description of the servlet.
   */
  public String getServletInfo(){
    return "Connection Manager Servlet";
  }

  /**
   * listen on socket clientRequests, accept MMrequests, process them, 
   * and return MMreply objects. 
   */
  public void run() {
    boolean finished = false;
    // this loop is in fact an infinite loop. It could be made to terminate by
    // placing a 'true' in finished. 
    while(!finished)
      {
	try {
	    Socket aSocket = clientRequests.accept();
	    CMrequest request = new CMrequest();
	    request.fromStream(aSocket.getInputStream());

	    // now service the request, and send the reply back. 
	    CMreply reply = serveRequest(request);
	    if(reply != null)
	      reply.toStream(aSocket.getOutputStream());

	    aSocket.close();
	}
	// for now ignore the exception. Just receive the next connection. 
	catch(IOException ioe) {
	}
      }
  }

  /** 
   * establish a connection between two local ports 'fromPort' and 'toPort'. 
   * @param fromPort: ID - port to connect to 'toPort'
   * @param toPort: ID - port to connect to 'fromPort'
   * @param callerHost: String - hostname on which the requester of 
   *                             establishConnection() resides
   * @param callerPort: int - port on which 'callerHost' listens
   * @param pr: PortRepresentation - representation of 'fromPort', locally
   * @return CMreply - reply to be sent to the requesting entity
   */
  CMreply establishLocal(ID fromPort, ID toPort, String callerHost, 
			 int callerPort, PortRepresentation pr) {
    CMreply rett = null;
    // make a mock request, and send it to the method for servicing 
    // RECEIVE requests. 
    CMreply reply = receiveConnection(fromPort, toPort, hostName, 
				      portNumberConnections);

    // if everything is OK, receive the connection and pass it on to the port. 
    if(reply.getStatus() == CMreply.OK) {
      try {
	Socket connSocket = portConnections.accept();
	pr.giveSocket(connSocket);
	rett = new CMreply(CMreply.OK);
      }
      catch(IOException ioe) {
	rett = new CMreply(CMreply.ERROR, 
			   "connection not received on initial side");
      }
    }
    else
      rett = new CMreply(CMreply.ERROR, reply.getDescription());

    return rett;
  }

  /**
   * establish a connection between 'fromPort' and 'toPort', where 'toPort' 
   * is a remote port, and 'fromPort' is a local port. 
   * @param fromPort: ID - local port to connect to 'toPort'
   * @param toPort: ID - remote port to connect to 'fromPort'
   * @param destHost: String - hostname on which toPort resides
   * @param callerHost: String - hostname on which the requester of 
   *                             establishConnection() resides
   * @param destPort: int - port on which 'destHost' listens
   * @param callerPort: int - port on which 'callerHost' listens
   * @param pr: PortRepresentation - representation of 'fromPort' stored in 
   *                                 the local ConnectionManager
   * @return CMreply - reply to be sent to the requesting entity
   */
  CMreply establishRemote(ID fromPort, ID toPort, String destHost, 
			  String callerHost, int destPort, int callerPort, 
			  PortRepresentation pr) {
    CMreply rett = null;

    // first send a request to the remote connection manager
    CMrequest req = new CMrequest(CMrequest.RECEIVE, fromPort, toPort, 
				  hostName, portNumberConnections);
    try {
      Socket s = new Socket(destHost, destPort);
      req.toStream(s.getOutputStream());

      // receive the reply, and if the reply is OK
      CMreply reply = new CMreply();
      reply.fromStream(s.getInputStream());
      if(reply.getStatus() == CMreply.OK) {
	// receive the connection from the remote CM and pass it to the port. 
	Socket connSocket = portConnections.accept();
	pr.giveSocket(connSocket);
	rett = new CMreply(CMreply.OK);
      }
      else
	rett = new CMreply(CMreply.ERROR, reply.getDescription());
    }
    // no error recovery for now. 
    catch(IOException ioe) {
      rett = new CMreply(CMreply.ERROR, "IO error occured on initial side");
    }
    return rett;
  }

  /**
   * establish a connection between fromPort and toPort. toPort is at host
   * destHost, which is listening on port destPort. The caller of 
   * establishConnection() is located at host callerHost, port callerPort
   * @param fromPort: ID - local port to connect to 'toPort'
   * @param toPort: ID - remote port to connect to 'fromPort'
   * @param destHost: String - hostname on which toPort resides
   * @param callerHost: String - hostname on which the requester of 
   *                             establishConnection() resides
   * @param destPort: int - port on which 'destHost' listens for requests
   * @param callerPort: int - port on which 'callerHost' listens
   * @return CMreply - reply to be sent to the requesting entity
   */
  CMreply establishConnection(ID fromPort, ID toPort, String destHost, 
			      String callerHost, int destPort, 
			      int callerPort) {
    CMreply rett = null;
    PortRepresentation pr = (PortRepresentation) portList.get(fromPort);
    if(pr == null)
      rett = new CMreply(CMreply.ERROR, "Initial port not found");
    else if(pr.getConnected())
      rett = new CMreply(CMreply.ERROR, "Initial port already connected");
    else if(destHost.equals(hostName) && destPort == portNumberRequests)
      rett = establishLocal(fromPort, toPort, callerHost, callerPort, pr);
    else
      rett = establishRemote(fromPort, toPort, destHost, callerHost, 
			     destPort, callerPort, pr);
    
    return rett;
  }

  /**
   * receive a connection from remote port 'fromPort' to local port 'toPort'. 
   * the caller is 'callerHost', listening on 'callerPort'. 
   * @param fromPort: ID - remote port to connect to 'toPort'
   * @param toPort: ID - local port to connect to 'fromPort'
   * @param callerHost: String - hostname on which 'fromPort' resides
   * @param callerPort: int - port on which 'callerHost' listens
   * @return CMreply - reply to be sent to the requesting entity
   */
  CMreply receiveConnection(ID fromPort, ID toPort, String callerHost, 
			    int callerPort) {
    CMreply rett = null;
    PortRepresentation pr = (PortRepresentation)portList.get(toPort);
    if(pr == null)
      rett = new CMreply(CMreply.ERROR, "Target port not found");
    else if(pr.getConnected())
      rett = new CMreply(CMreply.ERROR, "Target port already connected");
    else {
      pr.establishConnection(callerHost, callerPort);
      rett = new CMreply(CMreply.OK);
    }
    return rett;
  }

  /**
   * serve the request. Just a giant switch statement
   * @param req: CMrequest being served
   * @return CMreply - reply for the request in req
   */
  CMreply serveRequest(CMrequest req) {
    CMreply reply = null;
    int command = req.getCommand();

    // call the apropriate method for each separate request
    switch(command) {
    case CMrequest.ESTABLISH: 
      reply = establishConnection(req.getFromPort(), req.getToPort(), 
				  req.getToHost(), req.getCallerHost(), 
				  req.getDestinationPort(), 
				  req.getCallerPort());
      break;
    case CMrequest.RECEIVE: 
      reply = receiveConnection(req.getFromPort(), req.getToPort(), 
				req.getCallerHost(), req.getCallerPort());
      break;
    default: 
      reply = new CMreply(CMreply.ERROR, "Command not recognized");
    }
    return reply;
  }

  /**
   * make an ID factory. Initialize it with the hostname and portnumber. 
   * This method is a static method since it will be invoked by individual
   * ports, which have no reference to the ConnectionManager
   */
  static void makeFactory() {
    /*
     * Not important whether multiple ConnectionManagers have the same
     * initial IDFactory string (i.e. multiple port IDs are identical), as
     * only ports in the local ConnectionManager need to be distinguished. 
     */
    try {
      InetAddress local = java.net.InetAddress.getLocalHost();
      String host = local.getHostName();
      factory = new IDFactory(host+":ConnectionManager");
    }
    catch(UnknownHostException uhe) {
      // use the exception itself to make the factory
      factory = new IDFactory(uhe+":ConnectionManager");
    }      
  }

  /**
   * A static method that will register a new port with a given description.
   * @param whichPort: a reference to the new port that will be registered.
   * @param description: a string that describes the type of the port.
   * @return ID: returns unique ID for the port that was registered.
   */
  /* Since in the prototype there is only one thread of control going around, 
     there's no need to guard the portList (at least) from multiple accesses. 
     In the future, such guards will be provided. */
  public static ID registerPort( Port whichPort, String description){
    // if first invocation, create portList and factory
    if(portList == null)
      portList = new Hashtable();
    if(factory == null)
      makeFactory();

    // now get port ID, make portRepresentation, and store all in portList
    ID portID = factory.getUniqueID();
    PortRepresentation pr = new PortRepresentation(whichPort, description);
    portList.put(portID, pr);

    // return the ID
    return portID;
  }

  /**
   * A static method that will deregister a port that will be destroyed
   * @param portID - ID of the port to deregister
   */
  public static void deRegisterPort(ID portID) {
    if( portList.containsKey(portID) ) 
      portList.remove(portID);
  }

  /* test stub
  public static void main(String args[]) {
    NullPort np = new NullPort();
    ID pID = np.getPortID();
    NullPort np1 = new NullPort();
    ID pID1 = np1.getPortID();
    PortRepresentation pr = null;

    // first for port 0
    System.out.println("For port 0, ID is: ");
    pID.toStream(System.out);
    pr = (PortRepresentation) portList.get(pID);
    System.out.println("Description is: " + pr.getDescription());
    System.out.println("Connected is: " + pr.getConnected());
    pr.giveSocket((Socket)null);

    // then for port 1
    System.out.println("For port 1, ID is: ");
    pID1.toStream(System.out);
    pr = (PortRepresentation) portList.get(pID1);
    System.out.println("Description is: " + pr.getDescription());
    System.out.println("Connected is: " + pr.getConnected());

    // finally for port 0 again
    System.out.println("For port 0, ID is: ");
    pID.toStream(System.out);
    pr = (PortRepresentation) portList.get(pID);
    System.out.println("Description is: " + pr.getDescription());
    System.out.println("Connected is: " + pr.getConnected());
  }
  */
}

