
package webflow.backend;

import java.io.*;
import java.util.*;

/**
* the SMreply class is used when sending requests to the SessionManager 
* class. It extends the Message class. It stores a fewitems in it: 
* - the status (one of an enumeration: OK or ERROR)
* - a module ID
* - a Vector of port IDs
* - a description (if an error occured)
* For the description of the communication protocol which SMreply
* implements look under communication protocols. 
*/
/* Public for now. Should not be public */
public class SMreply extends Message {
  /** 
  * an enumeration of available statuses that the SessionManager might 
  * return 
  */
  public static final int ERROR = 0, OK = 1, HTML = 2;

  // the reply carries: a status, an ID of the module, a Vector with port IDs, 
  // a description of the module, and a key to the viewer html (if any)
  int stat;

  /** */
  ID id;

  /** */
  Vector list;

  /** */
  String des;

  /** */
  int htmlKey;

  /** */
  String viewerWindowName = null;

  /** 
  * void constructor used by the receiving side to create an empty 
  * SMreply object to get the incomming message
  */
  public SMreply() {
    stat=0;
    des = null;
    id = null;
    list = null; 
 }
 
  /** 
  * constructor used to implement replies to CONNECT, RUN, STOP, and DESTROY
  * requests (that need only a status)
  * @param status - return status
  */
  public SMreply(int status) {
    stat = status;
  }

  /** 
  * constructor used to reply when an error occured
  * @param status - return status
  * @param description - description of error
  */
  public SMreply(int status, String description) {
    stat=status;
    des=description;
  }

  /**
  * constructor used to reply to the NEW_MODULE request
  * @param status - return status
  * @param moduleID - ID of the created module
  * @param portList - Vector of port IDs
  */
  public SMreply(int status, ID moduleID, Vector portList) {
    stat = status;
    id = moduleID;
    list = portList;
  }

  /**
  * constructor used to reply to the NEW_MODULE request when a viewer is 
  * created
  * @param status - return status
  * @param moduleID - ID of the created module
  * @param portList - Vector of port IDs
  * @param key - a key for the viewer html
  */
  public SMreply(int status, ID moduleID, Vector portList, int key) {
    stat = status;
    id = moduleID;
    list = portList;
    htmlKey = key;
  }

  /**
  * constructor used to reply to the NEW_MODULE request when a viewer is 
  * created
  * @param status - return status
  * @param moduleID - ID of the created module
  * @param portList - Vector of port IDs
  * @param key - a key for the viewer html
  * @param _WinName - A name of the Netscape window 
  */
  public SMreply(int status, ID moduleID, Vector portList, int key, String _WinName) 
  {
    stat = status;
    id = moduleID;
    list = portList;
    htmlKey = key;
    if(_WinName!=null) viewerWindowName = new String(_WinName);
  }

  /** 
  * return the status stored in the object
  * @return int - status returned by the object
  */
  public int getStatus() {
    return  stat;
  }

  /**
  * return the module ID stored in the object
  * @return ID containing the ID of a module
  */
  public ID getModuleID() {
    return id;
  }

  /**
  * return the list of ports stored in the object
  * @return Vector containing objects of class ID - the port IDs
  */
  public Vector getPortList() {
    return list;
  }

  /**
   * return the html string stored in the object
   * @return int containing the html key stored in the object. 
   */
  public int getHtmlKey() {
    return(htmlKey);
  }

  /**
  * return the error description stored in the object
  * @return String containing an error description
  */
  public String getDescription() {
    return des;
  }

  /**
  * read the SMreply from the input stream 'in'. 
  * @param in: an InputStream from which the representation for the 
  * 	       SMreply will be read. 
  * @return true if the SMreply was read, false if some error occured. 
  */
  public boolean fromStream(InputStream in) 
  {
    boolean done ;
    int line;
    try
      { 
	DataInputStream din = new DataInputStream(in);

	// item 1: status
	stat = din.readInt();

	// item 2: guarded for null id
	if (din.readBoolean())
	  { 
	    id = new ID();
	    id.fromStream(din);
	  }
	else 
	  { 
	    id = null;
	    done = false;
	  }

	// item 3: guarded for null des
	if (din.readBoolean())
	  {
	    int bufsize= din.readInt();
	    byte[] b = new byte[bufsize];
	    din.readFully(b,0,bufsize);
	    des = new String(b,0,0,bufsize);
	  }
        else des = null;

	// item 4: htmlKey
	htmlKey = din.readInt();

	// item 5: guarded for null, list of ports
	if (din.readBoolean())
	  {
	    int size = din.readInt();
	    list = new Vector(size);

	    for (int i=0;i<size;i++)
	      {
		ID id = new ID();
		id.fromStream(din);
		list.addElement(id);
	      } 
	  }
	else
	  list = null;

	done = true;
      }
    catch (IOException e)
      {
	id  = null;
	stat =0;
	done =false;
      }
    
    return done;
    
  }
  
  /**
  * write the SMreply to the output stream 'out'. 
  * @param out: an OutputStream to which the representation for the SMreply 
  * 		will be written. 
  * @return true if the SMreply was written, false if some error occured. 
  */
  public boolean toStream(OutputStream out) 
  {
    boolean done; 
    try
      {
	DataOutputStream dout  = new DataOutputStream(out);
	// item 1: status
        dout.writeInt(stat);

	// item 2: id, guarded for null
        if(id == null)
          dout.writeBoolean(false);
        else { 
	  dout.writeBoolean(true);
	  id.toStream(dout);
        }

	// item 3: des, guarded for null
        if (des==null)
          dout.writeBoolean(false);
        else
	  {
	    dout.writeBoolean(true);
	    dout.writeInt(des.length());
	    dout.writeBytes(des);
	  }
	
	// item 4: htmlKey, not guarded for null
	dout.writeInt(htmlKey);
	
	// item 5: ID list (of ports), guarded for null
	if (list==null)
          dout.writeBoolean(false);
	else
          {
	    dout.writeBoolean(true);
	    dout.writeInt(list.size());
	    for(Enumeration e = list.elements();e.hasMoreElements();)
	      {
		ID id = ((ID)e.nextElement());
		id.toStream(dout);
	      }
	  }

	dout.flush();   
	done = true;
      }
    catch (IOException e)
      { done =false;
      }
    
    return done;
  }           

  /** 
  * Sets the name of netscape window.
  * @param _WinName Window name
  */
  public void
  setViewerWinName(String _WinName)
  { viewerWindowName = new String(_WinName); }

  /** 
  * Gets the  name of netscape window.
  * @return Window name
  */
  public String 
  getViewerWinName()
  { return (viewerWindowName!=null? new String(viewerWindowName): null); }
  
  /* test stub */
  public static void main(String[] argv)
  {
    ID id  = new ID("blah1",3);
    Vector list = new Vector(3);
    ID id1 = new ID("blah2", 4);
    list.addElement(id1);
    list.addElement(id);
    SMreply req = new SMreply(SMreply.OK);
    SMreply req1 = new SMreply(SMreply.ERROR,"Cannot reply");

    if(argv[0].equals("give"))
      {
	System.out.println("stat:" + req.getStatus());
	System.out.println("stat:" + req1.getStatus());
	System.out.println("stat:" + req1.getDescription());
      }
    else if(argv[0].equals("from"))
      {
	SMreply  req2 = new SMreply(SMreply.OK, id, list);
	if(!req2.fromStream(System.in))
          System.out.println("error  occured");
	else
	  {
	    System.out.println("stat:" + req2.getStatus());
	    System.out.println("desc:" + req2.getDescription());
	    System.out.println("html:" + req2.getHtmlKey());
	    System.out.print("list size : ");
	    Vector list2 = req2.getPortList();
	    System.out.println(list2.size());
	    for(Enumeration e = list2.elements();e.hasMoreElements();)
	      {
		ID id2 = ((ID)e.nextElement());
		System.out.println("ID: Str: " +id2.uniqueStr + " int: " + id2.uniqueInt);
	      }
	  }
      }
    else if(argv[0].equals("to"))
      { 
	SMreply  req2 = new SMreply(SMreply.HTML, id, list, 122);
	req2.toStream(System.out);
      }
  }  
}






