package webflow.backend;

import java.net.*;
/**
 * the Port class is the base class from which will be derived all the kind of
 * ports that will be used. This class stores the ID of the port and implements
 * the initialization of a port. This class defines two abstract methods, send
 * and receive, wich  must be overriten by each derived class. These methods
 * will specify the serialization and deserialization of an object that will 
 * be manipulated by the port. 
 */
public abstract class Port extends Object {

  // ID of the port
  ID portID;
  // Name of child class that is instantiated
  String childName;
  // communication socket
  private Socket s;

  /** 
   * Void constructor for class Port. It creates the port. It finds out the 
   * name of the child class that is instantiated, and uses that name to 
   * register itself with the ConnectionManager. 
   */
  public
  Port() {
    s = null;
    childName = this.getClass().getName();
    portID = ConnectionManager.registerPort(this, childName);
  }

  /**
   * Method called by ConnectionManager when a connection is received
   * The getPortID() and receiveConnection are synchronized, as they might
   * be called in the same time by a ConnectionManager and a port.  
   * @param connectionSocket : Socket over which the class will communicate
   */
  public final synchronized void receiveConnection(Socket connectionSocket) {
    s = connectionSocket;
  }

  /**
   * Return ID of the port.
   * The getPortID() and receiveConnection are synchronized, as they might
   * be called in the same time by a ConnectionManager and a port.  
   * @return ID - ID of the port, as given by the ConnectionManager. 
   */
  public final synchronized ID getPortID() {
    return portID;
  }

  /**
   * Return the communication socket
   * @return Socket - socket through which the child class will communicate
   */
  public final Socket getSocket() {
    return s;
  }

  /**
   * deregister the port from the ConnectionManager's list
   */
  public final void destroyPort() {
    ConnectionManager.deRegisterPort(portID);
  }


  /**
   * abstract method, which must be overriten by the derived classes. It 
   * should implement the serialization of an Object that the port will 
   * send.
   * @param o: object that will be send with the port 
   */
  public abstract void send( Object o );
  

  /**
   * abstract method, wich must be overriten by the derived classes. It should
   * implement the deserialization of an Object that the port will receive.
   * @return Object - returns the object that the port received. 
   */
  public abstract Object receive();

}

