package webflow.util.Workers;

import webflow.util.Action;
import java.lang.Runnable;

/**
 * Objects of class Worker are used to run objects that inherit the Action
 * interface. One separate thread exists over each Worker object, 
 * so that when a client gives the Worker an Action to run, 
 * the client immediately gets the control back, and the Action
 * is asynchronously run in the Worker thread. <BR>
 * At termination of the action Worker performs all the necessary clean up
 * (i.e. notifying the WorkerPool that it is now available, etc.) <BR>
 * Worker extends Thread so that it will have control over sleeping, yielding, 
 * etc. 
 * 
 * @version 0.1.0 first draft, April 21, 1997
 * @version 0.1   completed , August 11, 1997 (Hasan Timucin OZDEMIR)
 */
public class 
Worker extends Thread 
{
  /** 
  * which worker pool to notify when the worker finishes 
  * serving an action
  */
  WorkerPool pool;

  /** storage for the worker's action */
  ActionStorage storage;

  /**
  * Creates a Worker that will be stored in the WorkerPool. 
  * @param pool the WorkerPool which will be notified when the Worker 
  *  terminates its activity
  */
  public 
  Worker(WorkerPool pool) 
  {
    this.pool = pool;
    storage = new ActionStorage();
    setPriority(2);
    start();

  }// end of constructor

  /**
  * Implements the behavior of the separate thread executed over this object. 
  * This method will be ran as soon as the thread is created. It waits until
  * the thread is notified that it will serve an action, serves that action, 
  * and notifies the WorkerPool that it is now available. It repeats that
  * behavior forever. 
  */
  public synchronized void 
  run() 
  {

    while(true) 
    {

      /*****************************/
      /** see if there's an action */
      /*****************************/

      while(storage.checkForAction() == false) 
      {
         /**************************************/
         /** Wait until receving an 'notify()' */
         /**************************************/

         try 
         {  wait(); 
         } catch (InterruptedException ie) 
           { continue; }

      }// end of if

      /***************************************/
      /** Now it is time to serve the Action */
      /***************************************/
      Action a = storage.getAction();

      a.service();
      storage.setAction(null);

      /**************************************/
      /** Notify the WorkerPool object that */
      /** we cat take another order.        */
      /**************************************/
      try
      { pool.freeWorker(this);
      } catch (UnknownWorkerException uwe)
        {
          System.out.println("W weird one :"+uwe.getMessage());
        }

    }// end of while
    
  }// end of run()

  /**
  * Services a given Action. The Action will actually be executed in the run()
  * method. This only notifies the thread running over this object that it
  * has an Action to service. 
  * @param a Action to service
  */
  public synchronized void 
  service(Action a) 
  {
    /************************************/
    /** store the action in the storage */
    /************************************/

    storage.setAction(a);

    /*************************/
    /** Let's the game begin */
    /*************************/

    notify();

  }// end of service()

  public boolean
  isActionSet() 
  {  return  storage.checkForAction(); }

}// end of Worker Class


